package PPI::Statement::End;

# The useless stuff (although maybe containing POD) at the end of a file

use strict;
use UNIVERSAL 'isa';
use base 'PPI::Statement';

use vars qw{$VERSION};
BEGIN {
	$VERSION = '0.905';
}

1;
