package PPI::Structure::Constructor;

# The anonymous hash or array constructor

use strict;
use UNIVERSAL 'isa';
use base 'PPI::Structure';

use vars qw{$VERSION};
BEGIN {
	$VERSION = '0.905';
}

1;
